"""
Summary.

    Commons Module -- Common Functionality

"""
import os
import inspect
from pyaws.colors import Colors
from xlines import logger
from xlines.statics import local_config


try:
    from pyaws.core.oscodes_unix import exit_codes
    os_type = 'Linux'
    user_home = os.getenv('HOME')
    splitchar = '/'                             # character for splitting paths (linux)
    acct = Colors.ORANGE
    text = Colors.BRIGHT_PURPLE
    TITLE = Colors.WHITE + Colors.BOLD
except Exception:
    from pyaws.core.oscodes_win import exit_codes    # non-specific os-safe codes
    os_type = 'Windows'
    user_home = os.getenv('username')
    splitchar = '\\'                            # character for splitting paths (windows)
    acct = Colors.CYAN
    text = Colors.LT2GRAY
    TITLE = Colors.WHITE + Colors.BOLD


# universal colors
rd = Colors.RED + Colors.BOLD
yl = Colors.YELLOW + Colors.BOLD
fs = Colors.GOLD3
bd = Colors.BOLD
gn = Colors.BRIGHT_GREEN
title = Colors.BRIGHT_WHITE + Colors.BOLD
bbc = bd + Colors.BRIGHT_CYAN
frame = gn + bd
btext = text + Colors.BOLD
bwt = Colors.BRIGHT_WHITE
bdwt = Colors.BOLD + Colors.BRIGHT_WHITE
ub = Colors.UNBOLD
rst = Colors.RESET

# globals
container = []
config_dir = local_config['CONFIG']['CONFIG_PATH']
expath = local_config['EXCLUSIONS']['EX_EXT_PATH']
div = text + '/' + rst
div_len = 2
horiz = text + '-' + rst
arrow = bwt + '-> ' + rst
BUFFER = local_config['PROJECT']['BUFFER']


def linecount(path, whitespace=True):
    if whitespace:
        return len(open(path).readlines())
    return len(list(filter(lambda x: x != '\n', open(path).readlines())))


class ExcludedTypes():
    def __init__(self, ex_path, ex_container=[]):
        self.types = ex_container
        if not self.types:
            self.types.extend(self.parse_exclusions(ex_path))

    def excluded(self, path):
        for i in self.types:
            if i in path:
                return True
        return False

    def parse_exclusions(self, path):
        """
        Parse persistent fs location store for file extensions to exclude
        """
        try:
            return [x.strip() for x in open(path).readlines()]
        except OSError:
            return []


def print_header(w):
    total_width = w + local_config['PROJECT']['COUNT_COLUMN_WIDTH']
    header_lhs = 'object'
    header_rhs = 'line count'
    tab = '\t'.expandtabs(total_width - len(header_lhs) - len(header_rhs))
    tab4 = '\t'.expandtabs(4)
    print(tab4 + (horiz * (total_width)))
    print(f'{tab4}{header_lhs}{tab}{header_rhs}')
    print(tab4 + (horiz * (total_width)))


def print_footer(total, object_count, w):
    total_width = w + local_config['PROJECT']['COUNT_COLUMN_WIDTH']
    msg = 'Total ({} objects):'.format(str(object_count))
    tab = '\t'.expandtabs(total_width - len(msg) - len(str(total)) - 1)

    # redefine with color codes added
    msg = f'Total ({title + "{:,}".format(object_count) + rst} objects):'
    tab4 = '\t'.expandtabs(4)
    print(tab4 + (horiz * (total_width)))
    print(f'{tab4}{msg}{tab}{bd + "{:,}".format(total) + rst:>6}' + '\n')


def remove_duplicates(duplicates):
    """
    Summary.

        Module function utilsing a generator to remove duplicates
        from large scale lists with minimal resource use

    Args:
        duplicates (list): contains repeated elements

    Returns:
        generator object (iter)
    """
    uniq = []

    def dedup(d):
        for element in d:
            if element not in uniq:
                uniq.append(element)
                yield element
    return [x for x in dedup(duplicates)]


def remove_illegal(d, illegal):
    """
        Removes excluded file types

    Args:
        :d (list): list of filesystem paths ending with object
        :illegal (list):  list of file type extensions for ommission

    Returns:
        legal filesystem paths (str)
    """
    def parse_list(path):
        """Reads in list from file object"""
        with open(path) as f1:
            return [x.strip() for x in f1.readlines()]

    bad = []

    try:
        illegal_dirs = parse_list(local_config['EXCLUSIONS']['EX_DIR_PATH'])
    except KeyError:
        illegal_dirs = ['pycache', 'venv']

    for fpath in d:

        # filter for illegal file extensions
        fobject = os.path.split(fpath)[1]
        if ('.' in fobject) and ('.' + fobject.split('.')[1] in illegal):
            bad.append(fpath)

        # filter for illegal dirs
        elif list(filter(lambda x: x in fpath, illegal_dirs)):
            bad.append(fpath)
    return sorted(list(set(d) - set(bad)))


def locate_fileobjects(origin, path=expath):
    """
    Summary.

        - Walks local fs directories identifying all git repositories

    Args:
        - origin (str): filesystem directory location

    Returns:
        - paths, TYPE: list
        - Format:

         .. code-block:: json

                [
                    '/cloud-custodian/tools/c7n_mailer/c7n_mailer/utils_email.py',
                    '/cloud-custodian/tools/c7n_mailer/c7n_mailer/slack_delivery.py',
                    '/cloud-custodian/tools/c7n_mailer/c7n_mailer/datadog_delivery.py',
                    '/cloud-custodian/tools/c7n_sentry/setup.py',
                    '/cloud-custodian/tools/c7n_sentry/test_sentry.py',
                    '/cloud-custodian/tools/c7n_kube/setup.py',
                    '...
                ]

    """
    fobjects = []

    if os.path.isfile(origin):
        return [origin]

    for root, dirs, files in os.walk(origin):
        for file in [f for f in files if '.git' not in root]:
            try:

                full_path = os.path.abspath(os.path.join(root, file))

                #if not ex.excluded(full_path):
                fobjects.append(full_path)

            except OSError:
                logger.exception(
                    '%s: Read error while examining local filesystem path (%s)' %
                    (inspect.stack()[0][3], path)
                )
                continue
    return remove_duplicates(fobjects)



"""

Help Menu
    Help menu object containing body of help content.
    For printing with formatting

"""

from xlines.statics import PACKAGE
from xlines import Colors


PKG_ACCENT = Colors.ORANGE
PARAM_ACCENT = Colors.WHITE


synopsis_cmd = (
    Colors.RESET + PKG_ACCENT + PACKAGE +
    PARAM_ACCENT + ' --sum ' + Colors.RESET + ' <dir1> <dir2> <fname1> ... '
    )

url_doc = Colors.URL + 'http://xlines.readthedocs.io' + Colors.RESET
url_sc = Colors.URL + 'https://github.com/fstab50/xlines' + Colors.RESET

menu_body = Colors.BOLD + """
  DESCRIPTION""" + Colors.RESET + """

            Count lines of text. Utility for code projects
            Source Code:  """ + url_sc + """
    """ + Colors.BOLD + """
  SYNOPSIS""" + Colors.RESET + """

            """ + synopsis_cmd + """

                        -s, --sum <value1>...
                       [-e, --exclusions ]
                       [-c, --configure  ]
                       [-d, --debug    ]
                       [-h, --help     ]
                       [-m, --multiprocess  ]
                       [-w, --whitespace  ]
                       [-V, --version  ]
    """ + Colors.BOLD + """
  OPTIONS
        -s, --sum""" + Colors.RESET + """ (string): Sum the counts of all lines contained
            in filesystem objects referenced in the sum parameter
    """ + Colors.BOLD + """
        -c, --configure""" + Colors.RESET + """:  Configure runtime parameter via the cli
            menu. Change display format, color scheme, etc values
    """ + Colors.BOLD + """
        -e, --exclusions""" + Colors.RESET + """:  Print out list of file type extensions
            and directories excluded from line count calculations
    """ + Colors.BOLD + """
        -d, --debug""" + Colors.RESET + """: Print out additional debugging information
    """ + Colors.BOLD + """
        -h, --help""" + Colors.RESET + """: Show this help message and exit
    """ + Colors.BOLD + """
        -V, --version""" + Colors.RESET + """: Print package version and copyright info

    """



"""
Summary:
    Color code shortcut mapping class library

Module Classes:
    :ColorObject:  Packaging class for single color code & assoc attributes
    :ColorMap:  Builds map of ColorObject objects indexed by shortcut names
    :ColorMapper:  Wrapper class for ColorMap; calling class for entire chain

"""

from pyaws import Colors

# color object
co = Colors()


class ColorObject():
    def __init__(self, name, hexcode, description=''):
        self.name = name
        self.code = hexcode
        self.description = description

    def __repr__(self):
        """Returns a hexcode str when object printed"""
        return self.code


class ColorMap():
    """
    Class providing color index based on short attribute names
    """
    def __init__(self):
        """
        Color shortcuts defined here as class attributes
        """
        self.act = co.ORANGE
        self.accent = co.ORANGE
        self.aqu = co.AQUA
        self.bbc = co.BOLD + co.BRIGHT_CYAN
        self.bd = co.BOLD
        self.bbl = co.BRIGHT_BLUE
        self.bgn = co.BRIGHT_GREEN
        self.borg = co.BOLD + co.ORANGE
        self.bpl = co.BRIGHT_PURPLE
        self.brd = co.BOLD + co.RED
        self.bwt = co.BRIGHT_WHITE
        self.bdwt = co.BOLD + co.BRIGHT_WHITE
        self.btext = co.BOLD + co.BRIGHT_CYAN
        self.byl = co.BOLD + co.BRIGHT_YELLOW
        self.byg = co.BRIGHT_YELLOWGREEN
        self.dbl = co.DARK_BLUE
        self.dcy = co.DARK_CYAN
        self.dg1 = co.DARK_GRAY1
        self.dg2 = co.DARK_GRAY2
        self.highlight = co.BRIGHT_YELLOW2
        self.fs = co.GOLD3
        self.filesysystem = co.GOLD3
        self.gray = co.LT2GRAY
        self.text = co.BRIGHT_CYAN
        self.frame = co.BRIGHT_GREEN
        self.org = co.ORANGE
        self.rd = co.RED
        self.ub = co.UNBOLD
        self.wtgr = co.WHITE_GRAY
        self.yl = co.YELLOW
        self.rst = co.RESET

    def code(self, color):
        return getattr(self, color)

    def search(self, color):
        """
        Returns:
            any attribute names matching search color, TYPE: list
        """
        return [x for x in dir(self) if (color in x) and (not x.startswith('__'))]

    def contents(self):
        """
        Returns the index; list of all color shortcuts, TYPE: list
        """
        return [{x: getattr(self, x)} for x in dir(self) if not x.startswith('__')]


class ColorAttributes():
    def __init__(self):
        self.cm = ColorMap()
        self.map = self.mapper()
        self.dlist = [x for x in dir(co) if not x.startswith('__')]
        self.shortcuts = self.generate_shortcuts()

    def mapper(self):
        """
        Summary:
            Use map method when attribute name is uncertain

        Returns:
            :attribute (hex code, TYPE: str)

        """
        m = {}
        for d in [{k: v} for k, v in type(co).__dict__.items() if not k.startswith('__')]:
            for hex_dict in [{x: getattr(self.cm, x)} for x in dir(self.cm) if not x.startswith('__')]:
                for k, v in d.items():
                    for hex_k, hex_v in hex_dict.items():
                        if hex_v == v:
                            m[hex_k] = ColorObject(name=hex_k, hexcode=v, description=k)
        return m

    def description(self, color):
        return self.map.get(color).description if self.map.get(color) else None

    def hex(self, color):
        return self.map.get(color).code

    def generate_shortcuts(self):
        """
        Summary:
            Parses all class attributes in base color class

        Returns:
            shortcut names, TYPE: list

        """
        return [''.join([a.lower() for a in y.split('_')]) for y in self.dlist]

    def contents(self):
        """
        Prints out color names highlighted in respective color codes
        """
        tab4 = '\t'.expandtabs(4)
        tab8 = '\t'.expandtabs(8)
        print('\n{}Color Names:'.format(tab4))
        return [print('{}{}{}'.format(tab8, v, k)) for k, v in self.map.items()][0]



import sys
import threading
import time

# progress bar
from prompt_toolkit.shortcuts import ProgressBar
from prompt_toolkit.styles import Style
from prompt_toolkit.shortcuts.progress_bar import formatters


tab = '\t'.expandtabs(8)

style = Style.from_dict({
    '': 'orange',
})


custom_formatters = [
    formatters.Text(tab),
    formatters.Label(suffix=': '),
    formatters.Bar(start=' |', end='|', sym_a='#', sym_b='#', sym_c='.'),
    formatters.Text(' '),
    formatters.Progress(),
    formatters.Text(' '),
    formatters.Percentage(),
    formatters.Text(' [elapsed: '),
    formatters.TimeElapsed(),
    formatters.Text(' left: '),
    formatters.TimeLeft(),
    formatters.Text(']'),
    formatters.Text(tab),
]


class ProgressBarThread(threading.Thread):
    def __init__(self, label='Working', delay=0.1, cycles=500):
        super(ProgressBarThread, self).__init__()
        self.label = label
        self.delay = delay  # interval between updates
        self.running = False
        self.end = int(cycles)

    def start(self):
        self.running = True
        super(ProgressBarThread, self).start()

    def run(self):
        with ProgressBar(style=style, formatters=custom_formatters) as pb:
            for i in pb(range(self.end), label=self.label):
                time.sleep(self.delay)
                if not self.running:
                    break

    def stop(self):
        self.running = False
        self.join()  # wait for run() method to terminate
        sys.stdout.write('\r' + len(self.label)*' ' + '\r')  # clean-up
        sys.stdout.flush()


class SpinnerThread(threading.Thread):
    def __init__(self, label='Working', delay=0.2):
        super(ProgressBarThread, self).__init__()
        self.label = label
        self.delay = delay  # interval between updates
        self.running = False

    def start(self):
        self.running = True
        super(ProgressBarThread, self).start()

    def run(self):
        label = '\r' + self.label + ' '
        while self.running:
            for c in ('-', '\\', '|', '/'):
                sys.stdout.write(label + c)
                sys.stdout.flush()
                time.sleep(self.delay)

    def stop(self):
        self.running = False
        self.join()  # wait for run() method to terminate
        sys.stdout.write('\r' + len(self.label)*' ' + '\r')  # clean-up
        sys.stdout.flush()



#!/usr/bin/env python3
"""

#
# xlines, GPL v3 License
#
# Copyright (c) 2018-2019 Blake Huber
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the 'Software'), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
import os
import sys
import json
import inspect
import argparse
from pyaws.utils import stdout_message
from pyaws.colors import Colors
from xlines.statics import PACKAGE, local_config
from xlines.help_menu import menu_body
from xlines import about, logger
from xlines.mp import multiprocessing_main
from xlines.common import linecount, locate_fileobjects, remove_illegal
from xlines.common import ExcludedTypes
from xlines.colormap import ColorMap


cm = ColorMap()

try:
    from pyaws.core.oscodes_unix import exit_codes
    os_type = 'Linux'
    user_home = os.getenv('HOME')
    splitchar = '/'                             # character for splitting paths (linux)
    acct = Colors.ORANGE
    text = Colors.BRIGHT_PURPLE
    TITLE = Colors.WHITE + Colors.BOLD
except Exception:
    from pyaws.core.oscodes_win import exit_codes    # non-specific os-safe codes
    os_type = 'Windows'
    user_home = os.getenv('username')
    splitchar = '\\'                            # character for splitting paths (windows)
    acct = Colors.CYAN
    text = Colors.LT2GRAY
    TITLE = Colors.WHITE + Colors.BOLD

# universal colors
rd = Colors.RED + Colors.BOLD
yl = Colors.YELLOW + Colors.BOLD
fs = Colors.GOLD3
bd = Colors.BOLD
gn = Colors.BRIGHT_GREEN
title = Colors.BRIGHT_WHITE + Colors.BOLD
bbc = bd + Colors.BRIGHT_CYAN
frame = gn + bd
btext = text + Colors.BOLD
bwt = Colors.BRIGHT_WHITE
bdwt = Colors.BOLD + Colors.BRIGHT_WHITE
ub = Colors.UNBOLD
rst = cm.rst

# globals
container = []
config_dir = local_config['CONFIG']['CONFIG_PATH']
expath = local_config['EXCLUSIONS']['EX_EXT_PATH']
div = text + '/' + rst
div_len = 2
horiz = text + '-' + rst
arrow = bwt + '-> ' + rst
BUFFER = local_config['PROJECT']['BUFFER']


def _configure():
    """
        Add exclusions and update runtime constants

    Returns:
        Success | Failure, TYPE: bool
    """
    try:
        # clear screen
        os.system('cls' if os.name == 'nt' else 'clear')
        display_exclusions()

        with open(expath) as f1:
            exclusions = [x.strip() for x in f1.readlines()]

        # query user input for new exclusions
        response = input('  Enter file extension types to be excluded separated by commas [quit]: ')

        if not response:
            sys.exit(exit_codes['EX_OK']['Code'])
        else:
            add_list = response.split(',')

            # add new extensions to existing
            exclusions.extend(['.' + x for x in add_list if '.' not in x])

            # write out new exclusions config file
            with open(expath, 'w') as f2:
                f2.writelines([x + '\n' for x in exclusions])

            display_exclusions()    # display resulting exclusions set
            return True
    except OSError:
        stdout_message(
            message='Unable to modify local config file located at {}'.format(expath),
            prefix='WARN')
        return False


def display_exclusions():
    """
    Show list of all file type extensions which are excluded
    from line total calculations
    """
    msg = f'{expath} not found.  Run $ {PACKAGE} --configure'
    tab = '\t'.expandtabs(15)

    # numbering
    div = cm.bpl + ')' + rst

    try:

        if os.path.exists(expath):
            with open(expath) as f1:
                exclusions = [x.strip() for x in f1.readlines()]

        stdout_message(message='File types excluded from line totals:')

        for index, ext in enumerate(exclusions):
            print('{}{:>3}{}'.format(tab, index + 1, div + ':  ' + ext))

        sys.stdout.write('\n')
        return True

    except OSError as e:
        stdout_message(message=f'Error: {e}. ' + msg, prefix='WARN')
        return False


def sp_linecount(path, exclusions):
    """Single process line count"""
    p = path
    try:
        if os.path.isfile(path):
            return remove_illegal([path], exclusions)

        elif os.path.isdir(path):
            d = locate_fileobjects(path)
            valid_paths = remove_illegal(d, exclusions)
            return valid_paths
            #for p in valid_paths:
            #    print_reportline({p: linecount(p)})
    except UnicodeDecodeError:
        pass


def filter_args(kwarg_dict, *args):
    """
    Summary:
        arg, kwarg validity test

    Args:
        kwarg_dict: kwargs passed in to calling method or func
        args:  valid keywords for the caller

    Returns:
        True if kwargs are valid; else raise exception
    """
    # unpack if iterable passed in args - TBD (here)
    if kwarg_dict is not None:
        keys = [key for key in kwarg_dict]
        unknown_arg = list(filter(lambda x: x not in args, keys))
        if unknown_arg:
            raise KeyError(
                '%s: unknown parameter(s) provided [%s]' %
                (inspect.stack()[0][3], str(unknown_arg))
            )
    return True


def help_menu():
    """
    Displays help menu contents
    """
    tab = '\t'.expandtabs(22)
    print(
        Colors.BOLD + '\n' + tab + PACKAGE + Colors.RESET +
        ' help contents'
        )
    print(menu_body)
    return


def main(**kwargs):
    """ Main """
    keywords = ('root', 'debug', 'update', 'remediate')
    if filter_args(kwargs, *keywords):
        root = kwargs.get('root', user_home)
        update = kwargs.get('update', False)
        remediate = kwargs.get('remediate', False)
        debug = kwargs.get('debug', False)
    return False


def line_orchestrator(path):
    io_fail = []
    container = {}
    try:
        inc = linecount(path)
        fname = path.split('/')[-1]
        container[fname] = inc
    except Exception:
        io_fail.append(path)
    return container, io_fail


def longest_path(parameters, exclusions):
    """
        Traces all subdirectories of provided commandline paths
        using MaxWidth object

    Args:
        :parameters (list): list of all sys.argv parameters supplied with --sum
        :exclusions (ExcludedTypes object): types to exclude

    Returns:
        width (integer), number of characters in longest path
    """
    mp = MaxWidth()     # max path object

    for i in parameters:
        try:
            paths = sp_linecount(i, exclusions.types)
            width = mp.calc_maxpath(paths)
        except TypeError:
            stdout_message(message='Provided path appears to be invalid', prefix='WARN')
            sys.exit(0)
    return width


class MaxWidth():
    def __init__(self):
        self.buffer = local_config['PROJECT']['COUNT_COLUMN_WIDTH'] + BUFFER
        self.term_width = os.get_terminal_size().columns - self.buffer
        self.max_width = 0

    def calc_maxpath(self, path_list):
        for path in path_list:
            if len(path) > self.max_width:
                self.max_width = len(path)
            if len(path) > self.term_width:
                break
        return self.max_width if self.max_width < self.term_width else self.term_width


def options(parser, help_menu=False):
    """
    Summary:
        parse cli parameter options
    Returns:
        TYPE: argparse object, parser argument set
    """
    parser.add_argument("-e", "--exclusions", dest='exclusions', action='store_true', required=False)
    parser.add_argument("-C", "--configure", dest='configure', action='store_true', required=False)
    parser.add_argument("-d", "--debug", dest='debug', action='store_true', default=False, required=False)
    parser.add_argument("-h", "--help", dest='help', action='store_true', required=False)
    parser.add_argument("-m", "--multiprocess", dest='multiprocess', default=False, action='store_true', required=False)
    parser.add_argument("-s", "--sum", dest='sum', nargs='*', default='.', required=False)
    parser.add_argument("-w", "--whitespace", dest='whitespace', action='store_false', default=True, required=False)
    parser.add_argument("-V", "--version", dest='version', action='store_true', required=False)
    return parser.parse_args()


def package_version():
    """
    Prints package version and requisite PACKAGE info
    """
    print(about.about_object)
    sys.exit(exit_codes['EX_OK']['Code'])


def precheck():
    """
    Pre-execution Dependency Check
    """
    # check if git root dir set; otherwise use home
    # check logging enabled
    # check if config file
    return True


def print_header(w):
    total_width = w + local_config['PROJECT']['COUNT_COLUMN_WIDTH']
    header_lhs = 'object'
    header_rhs = 'line count'
    tab = '\t'.expandtabs(total_width - len(header_lhs) - len(header_rhs))
    tab4 = '\t'.expandtabs(4)
    print(tab4 + (horiz * (total_width)))
    print(f'{tab4}{header_lhs}{tab}{header_rhs}')
    print(tab4 + (horiz * (total_width)))


def print_footer(total, object_count, w):
    total_width = w + local_config['PROJECT']['COUNT_COLUMN_WIDTH']

    # add commas
    total_lines = '{:,}'.format(object_count)

    # calc dimensions; no color codes
    msg = 'Total ({} objects):'.format(total_lines)
    tab = '\t'.expandtabs(total_width - len(msg) - len(str(total)) - 1)

    # redefine with color codes added
    msg = f'Total ({title + "{:,}".format(object_count) + rst} objects):'
    tab4 = '\t'.expandtabs(4)

    # divider pattern
    print(tab4 + (horiz * (total_width)))

    # ending summary stats line
    print(f'{tab4}{msg}{tab}{bd + "{:,}".format(total) + rst:>6}' + '\n')


def init_cli():

    parser = argparse.ArgumentParser(add_help=False)

    try:
        args = options(parser)
    except Exception as e:
        help_menu()
        stdout_message(str(e), 'ERROR')
        sys.exit(exit_codes['E_BADARG']['Code'])

    if len(sys.argv) == 1:
        help_menu()
        sys.exit(exit_codes['EX_OK']['Code'])

    elif args.help:
        help_menu()
        sys.exit(exit_codes['EX_OK']['Code'])

    elif args.version:
        package_version()

    elif args.exclusions:
        display_exclusions()

    elif args.configure:
        _configure()

    elif args.sum and precheck():

        ex = ExcludedTypes(ex_path='/home/blake/.config/xlines/exclusions.list')
        container = []
        container.extend(args.sum)

        if args.debug:
            print('\nargs.sum:')
            print(args.sum)
            print('\nsys.argv contents:')
            print(sys.argv)

        if args.multiprocess:
            # --- run with concurrency --
            multiprocessing_main([x for x in args.sum], ex, args.debug)

        elif not args.multiprocess:

            io_fail = []
            tcount, tobjects = 0, 0
            width = longest_path(container, ex)

            print_header(width)
            count_width = local_config['PROJECT']['COUNT_COLUMN_WIDTH']
            hicount_threshold = local_config['PROJECT']['COUNT_THRESHOLD']

            for i in container:

                paths = sp_linecount(i, ex.types)

                for path in paths:
                    try:
                        inc = linecount(path, args.whitespace)
                        highlight = cm.accent if inc > hicount_threshold else cm.aqu
                        tcount += inc    # total line count
                        tobjects += 1    # increment total number of objects

                        # truncation
                        lpath = os.path.split(path)[0]
                        fname = os.path.split(path)[1]

                        if width < (len(lpath) + len(fname)):
                            cutoff = (len(lpath) + len(fname)) - width
                        else:
                            cutoff = 0

                        tab = '\t'.expandtabs(width - len(lpath) - len(fname) - count_width + BUFFER)
                        tab4 = '\t'.expandtabs(4)

                        # with color codes added
                        if cutoff == 0:
                            lpath = text + lpath + rst
                        else:
                            lpath = os.path.split(path)[0][:cutoff] + arrow

                        fname = highlight + fname + rst

                        # incremental count formatting
                        ct_format = acct if inc > hicount_threshold else bwt

                        output_str = f'{tab4}{lpath}{div}{fname}{tab}{ct_format}{"{:,}".format(inc):>7}{rst}'
                        print(output_str)
                    except Exception:
                        io_fail.append(path)
                        continue

            print_footer(tcount, tobjects, width)

            if args.debug:
                print('Skipped file objects:')
                for file in io_fail:
                    print('\t{}'.format(file))   # Write this out to a file in /tmp for later viewing
            sys.exit(exit_codes['E_DEPENDENCY']['Code'])

    else:
        stdout_message(
            'Dependency check fail %s' % json.dumps(args, indent=4),
            prefix='AUTH',
            severity='WARNING'
            )
        sys.exit(exit_codes['E_DEPENDENCY']['Code'])

    failure = """ : Check of runtime parameters failed for unknown reason.
    Please ensure you have both read and write access to local filesystem. """
    logger.warning(failure + 'Exit. Code: %s' % sys.exit(exit_codes['E_MISC']['Code']))
    print(failure)



"""
Summary.

    Multiprocessing module for line count

Module Functions:


"""
import os
import multiprocessing
from multiprocessing import Queue
from pyaws.utils import stdout_message, export_json_object
from pyaws.colors import Colors
from xlines.common import BUFFER, acct, bwt, text, rst, arrow, div
from xlines.statics import local_config
from xlines.common import locate_fileobjects, remove_illegal, linecount, print_header, print_footer


def longest_path_mp(object_list):
    longest = 0
    for path_dict in object_list:
        if len(path_dict['path']) > longest:
            longest = len(path_dict['path'])
    return longest


def mp_linecount(path, exclusions):
    """Multiprocessing line count"""
    p = path
    try:
        if os.path.isfile(path):
            q.put({
                    'path': os.path.abspath(path),
                    'count': linecount(path)
                }
            )

        elif os.path.isdir(path):
            d = locate_fileobjects(path)
            # remove paths to invalid objects in d
            for p in remove_illegal(d, exclusions):
                q.put({'path': p, 'count': linecount(p)})
    except UnicodeDecodeError:
        q.put({'path': p, 'count': None})
        return


def print_results(object_list):

    tcount, tobjects = 0, 0
    io_fail = []
    width = longest_path_mp(object_list)

    print_header(width)
    count_width = local_config['PROJECT']['COUNT_COLUMN_WIDTH']
    hicount_threshold = local_config['PROJECT']['COUNT_THRESHOLD']

    for object_dict in object_list:

        try:
            path = object_dict['path']
            inc = object_dict['count']
            highlight = acct if inc > hicount_threshold else Colors.AQUA
            tcount += inc    # total line count
            tobjects += 1    # increment total number of objects

            # truncation
            lpath = os.path.split(path)[0]
            fname = os.path.split(path)[1]

            if width < (len(lpath) + len(fname)):
                cutoff = (len(lpath) + len(fname)) - width
            else:
                cutoff = 0

            tab = '\t'.expandtabs(width - len(lpath) - len(fname) - count_width + BUFFER)
            tab4 = '\t'.expandtabs(4)

            # with color codes added
            if cutoff == 0:
                lpath = text + lpath + rst
            else:
                lpath = os.path.split(path)[0][:cutoff] + arrow

            fname = highlight + fname + rst

            # incremental count formatting
            ct_format = acct if inc > hicount_threshold else bwt

            output_str = f'{tab4}{lpath}{div}{fname}{tab}{ct_format}{"{:,}".format(inc):>7}{rst}'
            print(output_str)
        except Exception:
            io_fail.append(path)
            continue

    print_footer(tcount, tobjects, width)
    return True


def multiprocessing_main(container, exclusions, debug):
    """Execute Operations using concurrency (multi-process) model"""

    global q
    q = Queue()

    processes = []
    for i in container:
        t = multiprocessing.Process(target=mp_linecount, args=(i, exclusions.types))
        processes.append(t)
        t.start()

    for one_process in processes:
        one_process.join()

    results = []
    while not q.empty():
        results.append(q.get())

    print_results(results)

    if debug:
        export_json_object(results, logging=False)
        stdout_message(message='Num of objects: {}'.format(len(results)))
    return 0



"""
Summary:
    xlines Project-level Defaults and Settings

Module Attributes:
    - user_home (TYPE str):
        $HOME environment variable, present for most Unix and Unix-like POSIX systems
    - config_dir (TYPE str):
        directory name default for stsaval config files (.stsaval)
    - config_path (TYPE str):
        default for stsaval config files, includes config_dir (~/.stsaval)
    - key_deprecation (TYPE str):
        Deprecation logic that xlines uses when 2 keys exist for a user.
"""

import os
import inspect
import logging
from pyaws.script_utils import read_local_config, get_os, os_parityPath
from xlines._version import __version__
logger = logging.getLogger(__version__)
logger.setLevel(logging.INFO)


# --  project-level DEFAULTS  ------------------------------------------------


try:

    env_info = get_os(detailed=True)
    OS = env_info['os_type']
    user_home = env_info['HOME']

except KeyError as e:
    logger.critical(
        '%s: %s variable is required and not found in the environment' %
        (inspect.stack()[0][3], str(e)))
    raise e
else:
    # local vars -- this section executes as default; if windows, execute diff
    # section with appropriate pathnames

    # project
    PACKAGE = 'xlines'
    LICENSE = 'GPL v3'
    LICENSE_DESC = 'General Public License v3'
    version = __version__

    # config parameters
    config_dir = '.config'
    config_subdir = PACKAGE
    config_filename = 'xlinesconf.json'
    config_dirpath = user_home + '/' + config_dir + '/' + config_subdir
    config_path = config_dirpath + '/' + config_filename

    # output format
    count_column_width = 7
    min_buffer_chars = 6
    count_threshold = 1000

    # exclusions
    ext_filename = 'exclusions.list'
    dir_filename = 'directories.list'

    # logging parameters
    enable_logging = False
    log_mode = 'FILE'
    log_filename = 'xlines.log'
    log_dir = user_home + '/' + 'logs'
    log_path = log_dir + '/' + log_filename

    if OS == 'Windows':
        config_path = os_parityPath(config_path)
        log_path = os_parityPath(log_path)

    seed_config = {
        "PROJECT": {
            "PACKAGE": PACKAGE,
            "PACKAGE_VERSION": version,
            "HOME": user_home,
            "COUNT_COLUMN_WIDTH": count_column_width,
            "COUNT_THRESHOLD": count_threshold,
            "BUFFER":  min_buffer_chars,
        },
        "CONFIG": {
            "CONFIG_DATE": "",
            "CONFIG_FILENAME": config_filename,
            "CONFIG_DIR": config_dir,
            "CONFIG_SUBDIR": config_subdir,
            "CONFIG_PATH": config_path
        },
        "EXCLUSIONS": {
            "EX_FILENAME": ext_filename,
            "EX_EXT_PATH": config_dirpath + '/' + ext_filename,
            "EX_DIR_PATH": config_dirpath + '/' + dir_filename
        },
        "LOGGING": {
            "ENABLE_LOGGING": enable_logging,
            "LOG_FILENAME": log_filename,
            "LOG_PATH": log_path,
            "LOG_MODE": log_mode,
            "SYSLOG_FILE": False
        }
    }

try:
    if not os.path.exists(log_dir):
        os.mkdir(log_dir)
        os.chmod(log_dir, 0o755)
    if os.path.exists(config_path):
        # parse config file
        local_config = read_local_config(cfg=config_path)
        # fail to read, set to default config
        if not local_config:
            local_config = seed_config
    else:
        local_config = seed_config

except OSError as e:
    logger.exception(
        '%s: Error when attempting to access or create local log and config %s' %
        (inspect.stack()[0][3], str(e))
    )
    raise e



"""
Summary.

    Exclusions Module -- Display, Update, and Modification
    of objects to be excluded from line count totals

"""
import os
import inspect
from veryprettytable import VeryPrettyTable
from pyaws.colors import Colors
from xlines.colormap import ColorMap
from xlines.statics import local_config

cm = ColorMap()

try:
    from pyaws.core.oscodes_unix import exit_codes
    os_type = 'Linux'
    user_home = os.getenv('HOME')
    splitchar = '/'                             # character for splitting paths (linux)
    acct = cm.accent
    text = Colors.BRIGHT_PURPLE
except Exception:
    from pyaws.core.oscodes_win import exit_codes    # non-specific os-safe codes
    os_type = 'Windows'
    user_home = os.getenv('username')
    splitchar = '\\'                            # character for splitting paths (windows)
    acct = cm.bbc
    text = Colors.LT2GRAY


# universal colors
rd = Colors.RED + Colors.BOLD
yl = Colors.YELLOW + Colors.BOLD
fs = Colors.GOLD3
bd = cm.bd
gn = Colors.BRIGHT_GREEN
title = Colors.BRIGHT_WHITE + Colors.BOLD
bbc = bd + Colors.BRIGHT_CYAN
frame = gn + bd
btext = text + Colors.BOLD
bwt = Colors.BRIGHT_WHITE
bdwt = Colors.BOLD + Colors.BRIGHT_WHITE
ub = Colors.UNBOLD
rst = Colors.RESET

# globals
container = []
config_dir = local_config['CONFIG']['CONFIG_PATH']
expath = local_config['EXCLUSIONS']['EX_EXT_PATH']
BUFFER = local_config['PROJECT']['BUFFER']


class ExcludedTypes():
    def __init__(self, ex_path, ex_container=[]):
        self.types = ex_container
        if not self.types:
            self.types.extend(self.parse_exclusions(ex_path))

    def excluded(self, path):
        for i in self.types:
            if i in path:
                return True
        return False

    def parse_exclusions(self, path):
        """
        Parse persistent fs location store for file extensions to exclude
        """
        try:
            return [x.strip() for x in open(path).readlines()]
        except OSError:
            return []
